package com.benyanyi.okhttp.config;

import android.content.Context;

import com.benyanyi.okhttp.call.HttpCall;
import com.benyanyi.okhttp.call.Request;
import com.benyanyi.okhttp.download.DownloadCall;
import com.benyanyi.okhttp.download.DownloadRequest;
import com.benyanyi.okhttp.type.RequestType;
import com.benyanyi.okhttp.util.FormatUtil;
import com.benyanyi.okhttp.util.OkHttpLog;
import com.google.gson.Gson;

import java.io.File;
import java.util.Map;

import okhttp3.FormBody;
import okhttp3.MediaType;
import okhttp3.MultipartBody;
import okhttp3.RequestBody;

/**
 * @author YanYi
 * @date 2019-08-12 16:02
 * @email ben@yanyi.red
 * @overview
 */
public class HttpRequest implements RequestType {

    private final Context context;
    private final boolean isCache;
    private final String url;
    private String mCacheName;
    private okhttp3.Request request;

    public HttpRequest(Context context, String url, boolean isCache) {
        this.context = context;
        this.url = url;
        this.isCache = isCache;
        this.mCacheName = url;
    }

    @Override
    public Request getText(Map<String, Object> map) {
        StringBuilder str = new StringBuilder(url);
        if (FormatUtil.isMapNotEmpty(map)) {
            mCacheName = mCacheName + map;
            str.append("?");
            for (Map.Entry<String, Object> entry : map.entrySet()) {
                str.append(entry.getKey()).append("=").append(entry.getValue()).append("&");
            }
            str = new StringBuilder(str.substring(0, str.length() - 1));
        }
        OkHttpLog.d(str);
        request = new okhttp3.Request.Builder()
                .url(str.toString())
                .get()
                .build();
        return send();
    }

    @Override
    public Request postText(Map<String, Object> headerMap, Map<String, Object> bodyMap) {
        FormBody.Builder builder = new FormBody.Builder();
        if (FormatUtil.isMapNotEmpty(bodyMap)) {
            mCacheName = mCacheName + bodyMap;
            for (Map.Entry<String, Object> entry : bodyMap.entrySet()) {
                builder.add(entry.getKey(), entry.getValue().toString());
            }
        }
        FormBody build = builder.build();
        okhttp3.Request.Builder builder1 = new okhttp3.Request.Builder();
        if (FormatUtil.isMapNotEmpty(headerMap)) {
            for (Map.Entry<String, Object> entry : headerMap.entrySet()) {
                builder1.addHeader(entry.getKey(), entry.getValue().toString());
            }
        }
        request = builder1.url(url)
                .post(build)
                .build();
        return send();
    }

    @Override
    public Request postTextEncoded(Map<String, Object> headerMap, Map<String, Object> bodyMap) {
        FormBody.Builder builder = new FormBody.Builder();
        if (FormatUtil.isMapNotEmpty(bodyMap)) {
            mCacheName = mCacheName + bodyMap;
            for (Map.Entry<String, Object> entry : bodyMap.entrySet()) {
                builder.addEncoded(entry.getKey(), entry.getValue().toString());
            }
        }
        FormBody build = builder.build();
        okhttp3.Request.Builder builder1 = new okhttp3.Request.Builder();
        if (FormatUtil.isMapNotEmpty(headerMap)) {
            for (Map.Entry<String, Object> entry : headerMap.entrySet()) {
                builder1.addHeader(entry.getKey(), entry.getValue().toString());
            }
        }
        request = builder1.url(url)
                .post(build)
                .build();
        return send();
    }

    @Override
    public Request postMultipart(Map<String, Object> headerMap, Map<String, Object> bodyMap) {
        MultipartBody.Builder builder = new MultipartBody.Builder()
                .setType(MultipartBody.FORM);
        if (FormatUtil.isMapNotEmpty(bodyMap)) {
            for (Map.Entry<String, Object> entry : bodyMap.entrySet()) {
                if (entry.getValue() instanceof File) {
                    File file = (File) entry.getValue();
                    RequestBody requestBody = RequestBody.create(file, MediaType.parse("multipart/form-data"));
                    builder.addFormDataPart(entry.getKey(), file.getName(), requestBody);
                } else {
                    File file = new File(entry.getValue().toString());
                    if (file.exists()) {//判断是否为文件路径
                        RequestBody requestBody = RequestBody.create(file, MediaType.parse("multipart/form-data"));
                        builder.addFormDataPart(entry.getKey(), file.getName(), requestBody);
                    } else {
                        builder.addFormDataPart(entry.getKey(), entry.getValue().toString());
                    }
                }
            }
        }
        MultipartBody multipartBody = builder.build();
        okhttp3.Request.Builder headerBuilder = new okhttp3.Request.Builder();
        if (FormatUtil.isMapNotEmpty(headerMap)) {
            for (Map.Entry<String, Object> entry : headerMap.entrySet()) {
                headerBuilder.addHeader(entry.getKey(), entry.getValue().toString());
            }
        }
        request = new okhttp3.Request.Builder()
                .url(url)
                .post(multipartBody)
                .build();
        return send(true);
    }

    @Override
    public Request postJson(Map<String, String> headerMap, Map<String, Object> map) {
        MediaType mediaType = MediaType.parse("application/json; charset=utf-8");
        okhttp3.Request.Builder builder = new okhttp3.Request.Builder();
        if (FormatUtil.isMapNotEmpty(headerMap)) {
            for (Map.Entry<String, String> entry : headerMap.entrySet()) {
                builder.addHeader(entry.getKey(), entry.getValue());
            }
        }
        if (FormatUtil.isMapNotEmpty(map)) {
            mCacheName = mCacheName + map;
            RequestBody requestBody = RequestBody.create(new Gson().toJson(map), mediaType);
            request = builder
                    .url(url)
                    .post(requestBody)
                    .build();
        } else {
            request = builder
                    .url(url)
                    .post(new FormBody.Builder().build())
                    .build();
        }
        return send();
    }

    @Override
    public Request postJson(Map<String, String> headerMap, String jsonBody) {
        MediaType mediaType = MediaType.parse("application/json; charset=utf-8");
        okhttp3.Request.Builder builder = new okhttp3.Request.Builder();
        if (FormatUtil.isMapNotEmpty(headerMap)) {
            for (Map.Entry<String, String> entry : headerMap.entrySet()) {
                builder.addHeader(entry.getKey(), entry.getValue());
            }
        }
        if (FormatUtil.isNotEmpty(jsonBody)) {
            mCacheName = mCacheName + jsonBody;
            RequestBody requestBody = RequestBody.create(jsonBody, mediaType);
            request = builder
                    .url(url)
                    .post(requestBody)
                    .build();
        } else {
            request = builder
                    .url(url)
                    .post(new FormBody.Builder().build())
                    .build();
        }
        return send();
    }

    @Override
    public Request xml(Map<String, String> headerMap, String xmlBody) {
        String str = xmlBody;
        MediaType mediaType = MediaType.parse("text/xml; charset=UTF-8");
        if (FormatUtil.isNotEmpty(str)) {
            mCacheName = mCacheName + str;
        } else {
            str = "";
        }
        okhttp3.Request.Builder builder = new okhttp3.Request.Builder();
        if (FormatUtil.isMapNotEmpty(headerMap)) {
            for (Map.Entry<String, String> entry : headerMap.entrySet()) {
                builder.addHeader(entry.getKey(), entry.getValue());
            }
        }
        request = builder
                .url(url)
                .post(RequestBody.create(xmlBody, mediaType))
                .build();
        return send();
    }

    @Override
    public DownloadRequest download() {
        return new DownloadCall.Builder().setContext(context).setUrl(url).setCover(false).setSuffix("").builder();
    }

    @Override
    public DownloadRequest download(boolean isCover) {
        return new DownloadCall.Builder().setContext(context).setUrl(url).setCover(isCover).setSuffix("").builder();
    }

    @Override
    public DownloadRequest download(String suffix) {
        return new DownloadCall.Builder().setContext(context).setUrl(url).setCover(false).setSuffix(suffix).builder();
    }

    @Override
    public DownloadRequest download(boolean isCover, String suffix) {
        return new DownloadCall.Builder().setContext(context).setUrl(url).setCover(isCover).setSuffix(suffix).builder();
    }

    private Request send() {
        return send(false);
    }

    private Request send(boolean isFile) {
        return new HttpCall.Builder()
                .setCache(isCache)
                .setCacheUrl(mCacheName)
                .setContext(context)
                .setHttpClient(Client.getClient(context, isCache, isFile))
                .setRequest(request)
                .builder();
    }

}
