package com.benyanyi.okhttp.type;

import com.benyanyi.okhttp.call.Request;
import com.benyanyi.okhttp.download.DownloadRequest;

import java.util.Map;

/**
 * @author YanYi
 * @date 2019-08-12 16:09
 * @email ben@yanyi.red
 * @overview
 */
public interface RequestType {

    /**
     * get文本请求
     *
     * @param map 请求文本
     */
    Request getText(Map<String, Object> map);

    /**
     * post文本请求
     * 对请求体进行URL编码
     *
     * @param headerMap 请求头
     * @param bodyMap   请求体
     */
    Request postText(Map<String, Object> headerMap, Map<String, Object> bodyMap);

    /**
     * post文本请求
     * 不对请求体进行URL编码
     *
     * @param headerMap 请求头
     * @param bodyMap   请求体
     */
    Request postTextEncoded(Map<String, Object> headerMap, Map<String, Object> bodyMap);

    /**
     * post多图文上传
     *
     * @param headerMap 请求头
     * @param bodyMap   请求体
     */
    Request postMultipart(Map<String, Object> headerMap, Map<String, Object> bodyMap);

    /**
     * post json字符上传
     *
     * @param headerMap 请求头
     * @param map       需要拼接成json的键值对
     */
    Request postJson(Map<String, String> headerMap, Map<String, Object> map);

    /**
     * post json字符上传
     *
     * @param headerMap 请求头
     * @param jsonBody  请求体
     */
    Request postJson(Map<String, String> headerMap, String jsonBody);

    /**
     * post xml字符上传
     *
     * @param headerMap 请求头
     * @param xmlBody   请求体
     */
    Request xml(Map<String, String> headerMap, String xmlBody);

    /**
     * 文件下载
     */
    DownloadRequest download();

    /**
     * 文件下载
     *
     * @param isCover 是否覆盖之前的文件
     */
    DownloadRequest download(boolean isCover);

    /**
     * 文件下载
     *
     * @param suffix 文件名后缀
     */
    DownloadRequest download(String suffix);

    /**
     * 文件下载
     *
     * @param isCover 是否覆盖之前的文件
     * @param suffix  文件名后缀
     */
    DownloadRequest download(boolean isCover, String suffix);

}
